@@ -51,7 +51,7 @@
 #include "bpf.h"
 #include "bpfb.h"
 
-void synth_one_frame(kiss_fft_cfg fft_inv_cfg, short buf[], MODEL *model, float Sn_[], float Pn[], int prede, float *de_mem, float gain);
+void synth_one_frame(codec2_fftr_cfg fftr_inv_cfg, short buf[], MODEL *model, float Sn_[], float Pn[], int prede, float *de_mem, float gain);
 void print_help(const struct option *long_options, int num_opts, char* argv[]);
 
 
@@ -65,19 +65,19 @@ int main(int argc, char *argv[])
 {
     FILE *fout = NULL;	/* output speech file                    */
     FILE *fin;		/* input speech file                     */
-    short buf[N];	/* input/output buffer                   */
-    float buf_float[N];
-    float buf_float_bpf[N];
-    float Sn[M];	/* float input speech samples            */
-    float Sn_pre[N];	/* pre-emphasised input speech samples   */
+    short buf[N_SAMP];	/* input/output buffer                   */
+    float buf_float[N_SAMP];
+    float Sn[M_PITCH];	/* float input speech samples            */
+    float Sn_pre[M_PITCH];	/* pre-emphasised input speech samples   */
     COMP  Sw[FFT_ENC];	/* DFT of Sn[]                           */
-    kiss_fft_cfg  fft_fwd_cfg;
-    kiss_fft_cfg  fft_inv_cfg;
-    float w[M];	        /* time domain hamming window            */
+    codec2_fft_cfg  fft_fwd_cfg;
+    codec2_fftr_cfg  fftr_fwd_cfg;
+    codec2_fftr_cfg  fftr_inv_cfg;
+    float w[M_PITCH];	        /* time domain hamming window            */
     COMP  W[FFT_ENC];	/* DFT of w[]                            */
     MODEL model;
-    float Pn[2*N];	/* trapezoidal synthesis window          */
-    float Sn_[2*N];	/* synthesised speech */
+    float Pn[2*N_SAMP];	/* trapezoidal synthesis window          */
+    float Sn_[2*N_SAMP];	/* synthesised speech */
     int   i,m;		/* loop variable                         */
     int   frames;
     float prev_Wo, prev__Wo, prev_uq_Wo;
@@ -98,8 +98,8 @@ int main(int argc, char *argv[])
     int prede = 0;
     float pre_mem = 0.0, de_mem = 0.0;
     float ak[order];
-    COMP  Sw_[FFT_ENC];
-    COMP  Ew[FFT_ENC];
+    // COMP  Sw_[FFT_ENC];
+    // COMP  Ew[FFT_ENC];
 
     int phase0 = 0;
     float ex_phase[MAX_AMP+1];
@@ -140,12 +140,14 @@ int main(int argc, char *argv[])
     float gain = 1.0;
     int   bpf_en = 0;
     int   bpfb_en = 0;
-    float bpf_buf[BPF_N+N];
+    float bpf_buf[BPF_N+N_SAMP];
     float lspmelvq_mse = 0.0;
-    int   amread;
-    FILE *fam;
+    int   amread, Woread;
+    FILE *fam = NULL, *fWo = NULL;
     int   awread;
-    FILE *faw;
+    FILE *faw = NULL;
+    int   hmread;
+    FILE *fhm = NULL;
 
     char* opt_string = "ho:";
     struct option long_options[] = {
@@ -181,7 +183,9 @@ int main(int argc, char *argv[])
         { "bpf", no_argument, &bpf_en, 1 },
         { "bpfb", no_argument, &bpfb_en, 1 },
         { "amread", required_argument, &amread, 1 },
+        { "hmread", required_argument, &hmread, 1 },
         { "awread", required_argument, &awread, 1 },
+        { "Woread", required_argument, &Woread, 1 },
         #ifdef DUMP
         { "dump", required_argument, &dump, 1 },
         #endif
@@ -190,12 +194,13 @@ int main(int argc, char *argv[])
     };
     int num_opts=sizeof(long_options)/sizeof(struct option);
     COMP Aw[FFT_ENC];
+    COMP H[MAX_AMP];
 
-    for(i=0; i<M; i++) {
+    for(i=0; i<M_PITCH; i++) {
 	Sn[i] = 1.0;
 	Sn_pre[i] = 1.0;
     }
-    for(i=0; i<2*N; i++)
+    for(i=0; i<2*N_SAMP; i++)
 	Sn_[i] = 0;
 
     prev_uq_Wo = prev_Wo = prev__Wo = TWO_PI/P_MAX;
@@ -213,7 +218,7 @@ int main(int argc, char *argv[])
     e = prev_e = 1;
     hpf_states[0] = hpf_states[1] = 0.0;
 
-    nlp_states = nlp_create(M);
+    nlp_states = nlp_create(M_PITCH);
 
     if (argc < 2) {
         print_help(long_options, num_opts, argv);
@@ -278,16 +283,28 @@ int main(int argc, char *argv[])
 		        optarg, strerror(errno));
                     exit(1);
                 }
+            } else if(strcmp(long_options[option_index].name, "Woread") == 0) {
+	        if ((fWo = fopen(optarg,"rb")) == NULL) {
+	            fprintf(stderr, "Error opening float Wo file: %s: %s.\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }
             } else if(strcmp(long_options[option_index].name, "amread") == 0) {
 	        if ((fam = fopen(optarg,"rb")) == NULL) {
 	            fprintf(stderr, "Error opening float Am file: %s: %s.\n",
 		        optarg, strerror(errno));
                     exit(1);
                 }
+            } else if(strcmp(long_options[option_index].name, "hmread") == 0) {
+	        if ((fhm = fopen(optarg,"rb")) == NULL) {
+	            fprintf(stderr, "Error opening float Hm file: %s: %s.\n",
+		        optarg, strerror(errno));
+                    exit(1);
+                }
             } else if(strcmp(long_options[option_index].name, "awread") == 0) {
 	        if ((faw = fopen(optarg,"rb")) == NULL) {
 	            fprintf(stderr, "Error opening float Aw file: %s: %s.\n",
-		        optarg, strerror(errno));
+                            optarg, strerror(errno));
                     exit(1);
                 }
 	    } else if(strcmp(long_options[option_index].name, "dump_pitch_e") == 0) {
@@ -390,8 +407,9 @@ int main(int argc, char *argv[])
 
     /* Initialise ------------------------------------------------------------*/
 
-    fft_fwd_cfg = kiss_fft_alloc(FFT_ENC, 0, NULL, NULL); /* fwd FFT,used in several places   */
-    fft_inv_cfg = kiss_fft_alloc(FFT_DEC, 1, NULL, NULL); /* inverse FFT, used just for synth */
+    fft_fwd_cfg = codec2_fft_alloc(FFT_ENC, 0, NULL, NULL); /* fwd FFT,used in several places   */
+    fftr_fwd_cfg = codec2_fftr_alloc(FFT_ENC, 0, NULL, NULL); /* fwd FFT,used in several places   */
+    fftr_inv_cfg = codec2_fftr_alloc(FFT_DEC, 1, NULL, NULL); /* inverse FFT, used just for synth */
     make_analysis_window(fft_fwd_cfg, w, W);
     make_synthesis_window(Pn);
     quantise_init();
@@ -425,17 +443,17 @@ int main(int argc, char *argv[])
 
     frames = 0;
     sum_snr = 0;
-    while(fread(buf,sizeof(short),N,fin)) {
+    while(fread(buf,sizeof(short),N_SAMP,fin)) {
 	frames++;
 
-	for(i=0; i<N; i++)
+	for(i=0; i<N_SAMP; i++)
 	    buf_float[i] = buf[i];
 
 	/* optionally filter input speech */
 
         if (prede) {
-           pre_emp(Sn_pre, buf_float, &pre_mem, N);
-           for(i=0; i<N; i++)
+           pre_emp(Sn_pre, buf_float, &pre_mem, N_SAMP);
+           for(i=0; i<N_SAMP; i++)
                 buf_float[i] = Sn_pre[i];
         }
 
@@ -447,22 +465,22 @@ int main(int argc, char *argv[])
             /* BPF speech */
 
             for(i=0; i<BPF_N; i++)
-                bpf_buf[i] =  bpf_buf[N+i];
-            for(i=0; i<N; i++)
+                bpf_buf[i] =  bpf_buf[N_SAMP+i];
+            for(i=0; i<N_SAMP; i++)
                 bpf_buf[BPF_N+i] = buf_float[i];
             if (bpfb_en)
-                inverse_filter(&bpf_buf[BPF_N], bpfb, N, buf_float, BPF_N);
+                inverse_filter(&bpf_buf[BPF_N], bpfb, N_SAMP, buf_float, BPF_N);
             else
-                inverse_filter(&bpf_buf[BPF_N], bpf, N, buf_float, BPF_N);
+                inverse_filter(&bpf_buf[BPF_N], bpf, N_SAMP, buf_float, BPF_N);
         }
 
         /* shift buffer of input samples, and insert new samples */
 
-	for(i=0; i<M-N; i++) {
-	    Sn[i] = Sn[i+N];
+	for(i=0; i<M_PITCH-N_SAMP; i++) {
+	    Sn[i] = Sn[i+N_SAMP];
 	}
-	for(i=0; i<N; i++) {
-	    Sn[i+M-N] = buf_float[i];
+	for(i=0; i<N_SAMP; i++) {
+	    Sn[i+M_PITCH-N_SAMP] = buf_float[i];
         }
 
 	/*------------------------------------------------------------*\
@@ -471,7 +489,7 @@ int main(int argc, char *argv[])
 
 	\*------------------------------------------------------------*/
 
-	nlp(nlp_states,Sn,N,P_MIN,P_MAX,&pitch,Sw,W,&prev_uq_Wo);
+	nlp(nlp_states,Sn,N_SAMP,P_MIN,P_MAX,&pitch,Sw,W,&prev_uq_Wo);
 	model.Wo = TWO_PI/pitch;
 
 	dft_speech(fft_fwd_cfg, Sw, Sn, w);
@@ -510,7 +528,7 @@ int main(int argc, char *argv[])
 	\*------------------------------------------------------------*/
 
 	if (phase0) {
-	    float Wn[M];		        /* windowed speech samples */
+	    float Wn[M_PITCH];		        /* windowed speech samples */
 	    float Rk[order+1];                  /* autocorrelation coeffs  */
             COMP a[FFT_ENC];
 
@@ -520,14 +538,18 @@ int main(int argc, char *argv[])
 
 	    /* find aks here, these are overwritten if LPC modelling is enabled */
 
-            for(i=0; i<M; i++)
+            for(i=0; i<M_PITCH; i++)
                 Wn[i] = Sn[i]*w[i];
-	    autocorrelate(Wn,Rk,M,order);
+	    autocorrelate(Wn,Rk,M_PITCH,order);
 	    levinson_durbin(Rk,ak,order);
 
 	    /* determine voicing */
 
+	    #if 0
 	    snr = est_voicing_mbe(&model, Sw, W, Sw_, Ew);
+            #else
+	    snr = est_voicing_mbe(&model, Sw, W);
+            #endif
 
 	    if (dump_pitch_e)
 		fprintf(fjvm, "%f %f %d ", model.Wo, snr, model.voiced);
@@ -535,8 +557,8 @@ int main(int argc, char *argv[])
 	    //printf("snr %3.2f v: %d Wo: %f prev_Wo: %f\n", snr, model.voiced,
 	    //	   model.Wo, prev_uq_Wo);
             #ifdef DUMP
-	    dump_Sw_(Sw_);
-	    dump_Ew(Ew);
+	    //dump_Sw_(Sw_);
+	    //dump_Ew(Ew);
 	    dump_snr(snr);
             #endif
 
@@ -555,10 +577,11 @@ int main(int argc, char *argv[])
 
             for(i=0; i<=order; i++)
                 a[i].real = ak[i];
-            kiss_fft(fft_fwd_cfg, (kiss_fft_cpx *)a, (kiss_fft_cpx *)Aw);
+            codec2_fft(fft_fwd_cfg, a, Aw);
 
 	    if (hand_voicing) {
-		fscanf(fvoicing,"%d\n",&model.voiced);
+		int ret = fscanf(fvoicing,"%d\n",&model.voiced);
+                assert(ret == 1);
 	    }
 	}
 
@@ -584,7 +607,7 @@ int main(int argc, char *argv[])
 	    if (e < 0.0) {
 		int i;
 		FILE*f=fopen("x.txt","wt");
-		for(i=0; i<M; i++)
+		for(i=0; i<M_PITCH; i++)
 		    fprintf(f,"%f\n", Sn[i]);
 		fclose(f);
 		printf("e = %f frames = %d\n", e, frames);
@@ -756,6 +779,12 @@ int main(int argc, char *argv[])
             assert(ret == MAX_AMP);
         }
 
+        if (Woread) {
+            int ret = fread(&model.Wo, sizeof(float), 1, fWo);
+            model.L = floor(PI/model.Wo);
+            assert(ret == 1);
+        }
+
 	/*------------------------------------------------------------*\
 
           Synthesise and optional decimation to 20 or 40ms frame rate
@@ -797,7 +826,7 @@ int main(int argc, char *argv[])
             for(i=0; i<decimate; i++) {
                 if (lpc_model) {
                     lsp_to_lpc(&lsps_dec[i][0], &ak_dec[i][0], order);
-                    aks_to_M2(fft_fwd_cfg, &ak_dec[i][0], order, &model_dec[i], e_dec[i],
+                    aks_to_M2(fftr_fwd_cfg, &ak_dec[i][0], order, &model_dec[i], e_dec[i],
                               &snr, 0, simlpcpf, lpcpf, 1, LPCPF_BETA, LPCPF_GAMMA, Aw);
                     apply_lpc_correction(&model_dec[i]);
                     sum_snr += snr;
@@ -808,25 +837,28 @@ int main(int argc, char *argv[])
                     #endif
                 }
 
-                /* optionally rewad in Aw FFT vector, we really only care about the phase
-                   of each entry, used for reading in phases generated by Octave */
+                if (phase0) {
+                    /* optionally read in Aw, replacing values generated using LPC */
 
-                if (awread) {
-                    int j;
-                    int ret = fread(Aw, sizeof(COMP), FFT_ENC, faw);
-                    //for(j=0; j<10; j++) {
-                    //    fprintf(stderr, "%f %f\n", Aw[j].real, Aw[j].imag);
-                    //}
-                    //exit(0);
-                    assert(ret == FFT_ENC);
+                    if (awread) {
+                        int ret = fread(Aw, sizeof(COMP), FFT_ENC, faw);
+                        assert(ret == FFT_ENC);
+                    }
+                    
+                    /* optionally read in Hm directly, bypassing sampling of Aw[] */
+
+                    if (hmread) {
+                        int ret = fread(H, sizeof(COMP), MAX_AMP, fhm);
+                        assert(ret == MAX_AMP);
+                    } else {
+                        sample_phase(&model_dec[i], H, Aw);
+                    }
+                    phase_synth_zero_order(&model_dec[i], ex_phase, H);
                 }
-
-                if (phase0)
-                    phase_synth_zero_order(fft_fwd_cfg, &model_dec[i], ex_phase, Aw);
                 if (postfilt)
                     postfilter(&model_dec[i], &bg_est);
-                synth_one_frame(fft_inv_cfg, buf, &model_dec[i], Sn_, Pn, prede, &de_mem, gain);
-                if (fout != NULL) fwrite(buf,sizeof(short),N,fout);
+                synth_one_frame(fftr_inv_cfg, buf, &model_dec[i], Sn_, Pn, prede, &de_mem, gain);
+                if (fout != NULL) fwrite(buf,sizeof(short),N_SAMP,fout);
             }
 
             /* update memories for next frame ----------------------------*/
@@ -873,16 +905,16 @@ int main(int argc, char *argv[])
     return 0;
 }
 
-void synth_one_frame(kiss_fft_cfg fft_inv_cfg, short buf[], MODEL *model, float Sn_[],
+void synth_one_frame(codec2_fftr_cfg fftr_inv_cfg, short buf[], MODEL *model, float Sn_[],
                      float Pn[], int prede, float *de_mem, float gain)
 {
     int     i;
 
-    synthesise(fft_inv_cfg, Sn_, model, Pn, 1);
+    synthesise(fftr_inv_cfg, Sn_, model, Pn, 1);
     if (prede)
-        de_emp(Sn_, Sn_, de_mem, N);
+        de_emp(Sn_, Sn_, de_mem, N_SAMP);
 
-    for(i=0; i<N; i++) {
+    for(i=0; i<N_SAMP; i++) {
 	Sn_[i] *= gain;
 	if (Sn_[i] > 32767.0)
 	    buf[i] = 32767;
