@@ -36,7 +36,10 @@
 #include "quantise.h"
 #include "lpc.h"
 #include "lsp.h"
-#include "kiss_fft.h"
+#include "codec2_fft.h"
+#include "phase.h"
+#include "mbest.h"
+
 #undef PROFILE
 #include "machdep.h"
 
@@ -586,123 +589,6 @@ float lspmelvq_quantise(float *x, float *xq, int order)
   return mse;
 }
 
-#define MBEST_STAGES 4
-
-struct MBEST_LIST {
-    int   index[MBEST_STAGES];    /* index of each stage that lead us to this error */
-    float error;
-};
-
-struct MBEST {
-    int                entries;   /* number of entries in mbest list   */
-    struct MBEST_LIST *list;
-};
-
-
-static struct MBEST *mbest_create(int entries) {
-    int           i,j;
-    struct MBEST *mbest;
-
-    assert(entries > 0);
-    mbest = (struct MBEST *)malloc(sizeof(struct MBEST));
-    assert(mbest != NULL);
-
-    mbest->entries = entries;
-    mbest->list = (struct MBEST_LIST *)malloc(entries*sizeof(struct MBEST_LIST));
-    assert(mbest->list != NULL);
-
-    for(i=0; i<mbest->entries; i++) {
-	for(j=0; j<MBEST_STAGES; j++)
-	    mbest->list[i].index[j] = 0;
-	mbest->list[i].error = 1E32;
-    }
-
-    return mbest;
-}
-
-
-static void mbest_destroy(struct MBEST *mbest) {
-    assert(mbest != NULL);
-    free(mbest->list);
-    free(mbest);
-}
-
-
-/*---------------------------------------------------------------------------*\
-
-  mbest_insert
-
-  Insert the results of a vector to codebook entry comparison. The
-  list is ordered in order or error, so those entries with the
-  smallest error will be first on the list.
-
-\*---------------------------------------------------------------------------*/
-
-static void mbest_insert(struct MBEST *mbest, int index[], float error) {
-    int                i, j, found;
-    struct MBEST_LIST *list    = mbest->list;
-    int                entries = mbest->entries;
-
-    found = 0;
-    for(i=0; i<entries && !found; i++)
-	if (error < list[i].error) {
-	    found = 1;
-	    for(j=entries-1; j>i; j--)
-		list[j] = list[j-1];
-	    for(j=0; j<MBEST_STAGES; j++)
-		list[i].index[j] = index[j];
-	    list[i].error = error;
-	}
-}
-
-
-static void mbest_print(char title[], struct MBEST *mbest) {
-    int i,j;
-
-    fprintf(stderr, "%s\n", title);
-    for(i=0; i<mbest->entries; i++) {
-	for(j=0; j<MBEST_STAGES; j++)
-	    fprintf(stderr, "  %4d ", mbest->list[i].index[j]);
-	fprintf(stderr, " %f\n", mbest->list[i].error);
-    }
-}
-
-
-/*---------------------------------------------------------------------------*\
-
-  mbest_search
-
-  Searches vec[] to a codebbook of vectors, and maintains a list of the mbest
-  closest matches.
-
-\*---------------------------------------------------------------------------*/
-
-static void mbest_search(
-		  const float  *cb,     /* VQ codebook to search         */
-		  float         vec[],  /* target vector                 */
-		  float         w[],    /* weighting vector              */
-		  int           k,      /* dimension of vector           */
-		  int           m,      /* number on entries in codebook */
-		  struct MBEST *mbest,  /* list of closest matches       */
-		  int           index[] /* indexes that lead us here     */
-)
-{
-   float   e;
-   int     i,j;
-   float   diff;
-
-   for(j=0; j<m; j++) {
-	e = 0.0;
-	for(i=0; i<k; i++) {
-	    diff = cb[j*k+i]-vec[i];
-	    e += powf(diff*w[i],2.0);
-	}
-	index[0] = j;
-	mbest_insert(mbest, index, e);
-   }
-}
-
-
 /* 3 stage VQ LSP quantiser useing mbest search.  Design and guidance kindly submitted by Anssi, OH3GDD */
 
 float lspmelvq_mbest_encode(int *indexes, float *x, float *xq, int ndim, int mbest_entries)
@@ -729,7 +615,8 @@ float lspmelvq_mbest_encode(int *indexes, float *x, float *xq, int ndim, int mbe
   /* Stage 1 */
 
   mbest_search(codebook1, x, w, ndim, lspmelvq_cb[0].m, mbest_stage1, index);
-  //mbest_print("Stage 1:", mbest_stage1);
+  MBEST_PRINT("Stage 1:", mbest_stage1);
+
 
   /* Stage 2 */
 
@@ -739,7 +626,7 @@ float lspmelvq_mbest_encode(int *indexes, float *x, float *xq, int ndim, int mbe
 	  target[i] = x[i] - codebook1[ndim*n1+i];
       mbest_search(codebook2, target, w, ndim, lspmelvq_cb[1].m, mbest_stage2, index);
   }
-  //mbest_print("Stage 2:", mbest_stage2);
+  MBEST_PRINT("Stage 2:", mbest_stage2);
 
   /* Stage 3 */
 
@@ -750,7 +637,7 @@ float lspmelvq_mbest_encode(int *indexes, float *x, float *xq, int ndim, int mbe
 	  target[i] = x[i] - codebook1[ndim*n1+i] - codebook2[ndim*n2+i];
       mbest_search(codebook3, target, w, ndim, lspmelvq_cb[2].m, mbest_stage3, index);
   }
-  //mbest_print("Stage 3:", mbest_stage3);
+  MBEST_PRINT("Stage 3:", mbest_stage3);
 
   n1 = mbest_stage3->list[0].index[2];
   n2 = mbest_stage3->list[0].index[1];
@@ -846,13 +733,13 @@ void force_min_lsp_dist(float lsp[], int order)
 
 \*---------------------------------------------------------------------------*/
 
-void lpc_post_filter(kiss_fft_cfg fft_fwd_cfg, COMP Pw[], float ak[],
+void lpc_post_filter(codec2_fftr_cfg fftr_fwd_cfg, float Pw[], float ak[],
                      int order, int dump, float beta, float gamma, int bass_boost, float E)
 {
     int   i;
-    COMP  x[FFT_ENC];   /* input to FFTs                */
-    COMP  Ww[FFT_ENC];  /* weighting spectrum           */
-    float Rw[FFT_ENC];  /* R = WA                       */
+    float x[FFT_ENC];   /* input to FFTs                */
+    COMP  Ww[FFT_ENC/2+1];  /* weighting spectrum           */
+    float Rw[FFT_ENC/2+1];  /* R = WA                       */
     float e_before, e_after, gain;
     float Pfw;
     float max_Rw, min_Rw;
@@ -864,17 +751,16 @@ void lpc_post_filter(kiss_fft_cfg fft_fwd_cfg, COMP Pw[], float ak[],
     /* Determine weighting filter spectrum W(exp(jw)) ---------------*/
 
     for(i=0; i<FFT_ENC; i++) {
-	x[i].real = 0.0;
-	x[i].imag = 0.0;
+	x[i] = 0.0;
     }
 
-    x[0].real = ak[0];
+    x[0]  = ak[0];
     coeff = gamma;
     for(i=1; i<=order; i++) {
-	x[i].real = ak[i] * coeff;
+	x[i] = ak[i] * coeff;
         coeff *= gamma;
     }
-    kiss_fft(fft_fwd_cfg, (kiss_fft_cpx *)x, (kiss_fft_cpx *)Ww);
+    codec2_fftr(fftr_fwd_cfg, x, Ww);
 
     PROFILE_SAMPLE_AND_LOG(tfft2, taw, "        fft2");
 
@@ -888,7 +774,7 @@ void lpc_post_filter(kiss_fft_cfg fft_fwd_cfg, COMP Pw[], float ak[],
 
     max_Rw = 0.0; min_Rw = 1E32;
     for(i=0; i<FFT_ENC/2; i++) {
-	Rw[i] = sqrtf(Ww[i].real * Pw[i].real);
+	Rw[i] = sqrtf(Ww[i].real * Pw[i]);
 	if (Rw[i] > max_Rw)
 	    max_Rw = Rw[i];
 	if (Rw[i] < min_Rw)
@@ -909,7 +795,7 @@ void lpc_post_filter(kiss_fft_cfg fft_fwd_cfg, COMP Pw[], float ak[],
 
     e_before = 1E-4;
     for(i=0; i<FFT_ENC/2; i++)
-	e_before += Pw[i].real;
+	e_before += Pw[i];
 
     /* apply post filter and measure energy  */
 
@@ -922,8 +808,8 @@ void lpc_post_filter(kiss_fft_cfg fft_fwd_cfg, COMP Pw[], float ak[],
     e_after = 1E-4;
     for(i=0; i<FFT_ENC/2; i++) {
         Pfw = powf(Rw[i], beta);
-        Pw[i].real *= Pfw * Pfw;
-        e_after += Pw[i].real;
+        Pw[i] *= Pfw * Pfw;
+        e_after += Pw[i];
     }
     gain = e_before/e_after;
 
@@ -931,14 +817,14 @@ void lpc_post_filter(kiss_fft_cfg fft_fwd_cfg, COMP Pw[], float ak[],
 
     gain *= E;
     for(i=0; i<FFT_ENC/2; i++) {
-	Pw[i].real *= gain;
+	Pw[i] *= gain;
     }
 
     if (bass_boost) {
         /* add 3dB to first 1 kHz to account for LP effect of PF */
 
         for(i=0; i<FFT_ENC/8; i++) {
-            Pw[i].real *= 1.4*1.4;
+            Pw[i] *= 1.4*1.4;
         }
     }
 
@@ -957,7 +843,7 @@ void lpc_post_filter(kiss_fft_cfg fft_fwd_cfg, COMP Pw[], float ak[],
 \*---------------------------------------------------------------------------*/
 
 void aks_to_M2(
-  kiss_fft_cfg  fft_fwd_cfg,
+  codec2_fftr_cfg  fftr_fwd_cfg,
   float         ak[],	     /* LPC's */
   int           order,
   MODEL        *model,	     /* sinusoidal model parameters for this frame */
@@ -972,8 +858,6 @@ void aks_to_M2(
   COMP          Aw[]         /* output power spectrum */
 )
 {
-  COMP a[FFT_ENC];	/* input to FFT for power spectrum */
-  COMP Pw[FFT_ENC];	/* output power spectrum */
   int i,m;		/* loop variables */
   int am,bm;		/* limits of current band */
   float r;		/* no. rads/bin */
@@ -987,33 +871,49 @@ void aks_to_M2(
   r = TWO_PI/(FFT_ENC);
 
   /* Determine DFT of A(exp(jw)) --------------------------------------------*/
+  {
+      float a[FFT_ENC];  /* input to FFT for power spectrum */
 
-  for(i=0; i<FFT_ENC; i++) {
-    a[i].real = 0.0;
-    a[i].imag = 0.0;
-    Pw[i].real = 0.0;
-    Pw[i].imag = 0.0;
+      for(i=0; i<FFT_ENC; i++) {
+          a[i] = 0.0;
+      }
+
+      for(i=0; i<=order; i++)
+          a[i] = ak[i];
+      codec2_fftr(fftr_fwd_cfg, a, Aw);
   }
-
-  for(i=0; i<=order; i++)
-    a[i].real = ak[i];
-  kiss_fft(fft_fwd_cfg, (kiss_fft_cpx *)a, (kiss_fft_cpx *)Aw);
-
   PROFILE_SAMPLE_AND_LOG(tfft, tstart, "      fft");
 
   /* Determine power spectrum P(w) = E/(A(exp(jw))^2 ------------------------*/
 
+  float Pw[FFT_ENC/2];
+
+#ifndef ARM_MATH_CM4
   for(i=0; i<FFT_ENC/2; i++) {
-    Pw[i].real = 1.0/(Aw[i].real*Aw[i].real + Aw[i].imag*Aw[i].imag + 1E-6);
+    Pw[i] = 1.0/(Aw[i].real*Aw[i].real + Aw[i].imag*Aw[i].imag + 1E-6);
   }
+#else
+  // this difference may seem strange, but the gcc for STM32F4 generates almost 5 times
+  // faster code with the two loops: 1120 ms -> 242 ms
+  // so please leave it as is or improve further
+  // since this code is called 4 times it results in almost 4ms gain (21ms -> 17ms per audio frame decode @ 1300 )
+
+  for(i=0; i<FFT_ENC/2; i++)
+  {
+      Pw[i] = Aw[i].real * Aw[i].real + Aw[i].imag * Aw[i].imag  + 1E-6;
+  }
+  for(i=0; i<FFT_ENC/2; i++) {
+      Pw[i] = 1.0/(Pw[i]);
+  }
+#endif
 
   PROFILE_SAMPLE_AND_LOG(tpw, tfft, "      Pw");
 
   if (pf)
-      lpc_post_filter(fft_fwd_cfg, Pw, ak, order, dump, beta, gamma, bass_boost, E);
+      lpc_post_filter(fftr_fwd_cfg, Pw, ak, order, dump, beta, gamma, bass_boost, E);
   else {
-      for(i=0; i<FFT_ENC; i++) {
-          Pw[i].real *= E;
+      for(i=0; i<FFT_ENC/2; i++) {
+          Pw[i] *= E;
       }
   }
 
@@ -1034,10 +934,20 @@ void aks_to_M2(
   for(m=1; m<=model->L; m++) {
       am = (int)((m - 0.5)*model->Wo/r + 0.5);
       bm = (int)((m + 0.5)*model->Wo/r + 0.5);
+
+      // FIXME: With arm_rfft_fast_f32 we have to use this
+      // otherwise sometimes a to high bm is calculated
+      // which causes trouble later in the calculation
+      // chain
+      // it seems for some reason model->Wo is calculated somewhat too high
+      if (bm>FFT_ENC/2)
+      {
+          bm = FFT_ENC/2;
+      }
       Em = 0.0;
 
       for(i=am; i<bm; i++)
-          Em += Pw[i].real;
+          Em += Pw[i];
       Am = sqrtf(Em);
 
       signal += model->A[m]*model->A[m];
@@ -1057,7 +967,6 @@ void aks_to_M2(
           if (Am < model->A[m])
               Am *= 1.4;
       }
-
       model->A[m] = Am;
   }
   *snr = 10.0*log10f(signal/noise);
@@ -1261,12 +1170,12 @@ float speech_to_uq_lsps(float lsp[],
 )
 {
     int   i, roots;
-    float Wn[M];
+    float Wn[M_PITCH];
     float R[order+1];
     float e, E;
 
     e = 0.0;
-    for(i=0; i<M; i++) {
+    for(i=0; i<M_PITCH; i++) {
 	Wn[i] = Sn[i]*w[i];
 	e += Wn[i]*Wn[i];
     }
@@ -1279,7 +1188,7 @@ float speech_to_uq_lsps(float lsp[],
 	return 0.0;
     }
 
-    autocorrelate(Wn, R, M, order);
+    autocorrelate(Wn, R, M_PITCH, order);
     levinson_durbin(R, ak, order);
 
     E = 0.0;
@@ -1919,7 +1828,7 @@ float decode_energy(int index, int bits)
 
 \*---------------------------------------------------------------------------*/
 
-float decode_amplitudes(kiss_fft_cfg  fft_fwd_cfg,
+float decode_amplitudes(codec2_fft_cfg  fft_fwd_cfg,
 			MODEL *model,
 			float  ak[],
 		        int    lsp_indexes[],
